@@ -25,7 +25,7 @@
 ### the mle function of the stat package.
 
 mledist <- function (data, distr, start=NULL, fix.arg=NULL, optim.method="default", 
-    lower=-Inf, upper=Inf, custom.optim=NULL, ...)
+    lower=-Inf, upper=Inf, custom.optim=NULL, weights=NULL, silent=TRUE, ...)
     # data may correspond to a vector for non censored data or to
     # a dataframe of two columns named left and right for censored data 
 {
@@ -38,11 +38,19 @@ mledist <- function (data, distr, start=NULL, fix.arg=NULL, optim.method="defaul
     if (!exists(ddistname, mode="function"))
         stop(paste("The ", ddistname, " function must be defined"))
 
-    if (!is.null(fix.arg) & is.null(start))
-        stop("Starting values must be defined when some distribution parameters are fixed")
+    start.arg <- start #to avoid confusion with the start() function of stats pkg (check is done lines 87-100)
+    if(is.vector(start.arg)) #backward compatibility
+      start.arg <- as.list(start.arg)
     
     txt1 <- "data must be a numeric vector of length greater than 1 for non censored data"
     txt2 <- "or a dataframe with two columns named left and right and more than one line for censored data"
+    if(!is.null(weights))
+    {
+      if(any(weights < 0))
+        stop("weights should be a vector of numerics greater than 0")
+      if(length(weights) != NROW(data))
+        stop("weights should be a vector with a length equal to the observation number")
+    }
     
     if (is.vector(data)) {
         cens <- FALSE
@@ -76,84 +84,28 @@ mledist <- function (data, distr, start=NULL, fix.arg=NULL, optim.method="defaul
         data<-c(rcens, lcens, ncens, (icens$left+icens$right)/2)
     }
     
-    # MLE fit 
-    # definition of starting values if not previously defined
-    if (is.null(start)) {
-        if (distname == "norm") {
-            n <- length(data)
-            sd0 <- sqrt((n - 1)/n) * sd(data)
-            mx <- mean(data)
-            start <- list(mean=mx, sd=sd0)
-        }
-        if (distname == "lnorm") {
-            if (any(data <= 0)) 
-                stop("values must be positive to fit a lognormal distribution")
-            n <- length(data)
-            ldata <- log(data)
-            sd0 <- sqrt((n - 1)/n) * sd(ldata)
-            ml <- mean(ldata)
-            start <- list(meanlog=ml, sdlog=sd0)
-        }
-        if (distname == "pois") {
-            start <- list(lambda=mean(data))
-        }
-        if (distname == "exp") {
-            start <- list(rate=1/mean(data))
-        }
-        if (distname == "gamma") {
-            n <- length(data)
-            m <- mean(data)
-            v <- (n - 1)/n*var(data)
-            start <- list(shape=m^2/v, rate=m/v)
-        }
-        if (distname == "nbinom") {
-            n <- length(data)
-            m <- mean(data)
-            v <- (n - 1)/n*var(data)
-            size <- if (v > m) m^2/(v - m)
-                else 100
-            start <- list(size = size, mu = m) 
-        }
-        if (distname == "geom" ) {
-            m <- mean(data)
-            prob <- if (m>0) 1/(1+m)
-                    else 1
-            start <- list(prob=prob)        
-        }
-        if (distname == "beta") {
-            if (any(data < 0) | any(data > 1)) 
-                stop("values must be in [0-1] to fit a beta distribution")
-            n <- length(data)
-            m <- mean(data)
-            v <- (n - 1)/n*var(data)
-            aux <- m*(1-m)/v - 1
-            start <- list(shape1=m*aux, shape2=(1-m)*aux)
-        }
-        if (distname == "weibull") {
-            m <- mean(log(data))
-            v <- var(log(data))
-            shape <- 1.2/sqrt(v)
-            scale <- exp(m + 0.572/shape)
-            start <- list(shape = shape, scale = scale)
-        }
-        if (distname == "logis") {
-            n <- length(data)
-            m <- mean(data)
-            v <- (n - 1)/n*var(data)
-            start <- list(location=m, scale=sqrt(3*v)/pi)
-        }
-        if (distname == "cauchy") {
-            start <- list(location=median(data), scale=IQR(data)/2)
-        }
-        if (distname == "unif"){
-            start <- list(min=0, max=1)
-        }
-
-        if (!is.list(start)) 
-            stop("'start' must be defined as a named list for this distribution") 
-   } # end of the definition of starting values     
+    # definition of starting/fixed values values
+    argddistname <- names(formals(ddistname))
+    chfixstt <- checkparam(start.arg=start.arg, fix.arg=fix.arg, argdistname=argddistname, 
+                           errtxt=NULL, data10=head(data, 10), distname=distname)
+    if(!chfixstt$ok)
+      stop(chfixstt$txt)
+    #unlist starting values as needed in optim()
+    if(is.function(chfixstt$start.arg))
+      vstart <- chfixstt$start.arg(data)
+    else
+      vstart <- unlist(chfixstt$start.arg)
+    if(is.function(fix.arg)) #function
+    { 
+      fix.arg.fun <- fix.arg
+      fix.arg <- fix.arg(data)
+    }else
+      fix.arg.fun <- NULL
+    #otherwise fix.arg is a named list or NULL
     
-    ############# closed formula for uniform distribution ##########
+    # end of the definition of starting/fixed values   
+    
+    ############# closed-form formula for uniform distribution ##########
     if(distname == "unif")
     {
         par <- c(min=min(data), max=max(data))
@@ -164,24 +116,10 @@ mledist <- function (data, distr, start=NULL, fix.arg=NULL, optim.method="defaul
     
    
     ############# MLE fit using optim or custom.optim ##########
-    vstart <- unlist(start)
-    vfix.arg <- unlist(fix.arg)
-    # check of the names of the arguments of the density function
-    argddistname <- names(formals(ddistname))   
-    m <- match(names(start), argddistname)
-    mfix <- match(names(vfix.arg), argddistname)
-    if (any(is.na(m)) || length(m) == 0)
-        stop("'start' must specify names which are arguments to 'distr'")
-    if (any(is.na(mfix)))
-        stop("'fix.arg' must specify names which are arguments to 'distr'")
-    # check that some parameters are not both in fix.arg and start
-    minter <- match(names(start), names(fix.arg))
-    if (any(!is.na(minter)))
-        stop("a distribution parameter cannot be specified both in 'start' and 'fix.arg'")
 
     # definition of the function to minimize : - log likelihood
     # for non censored data
-    if (!cens) {
+    if (!cens && is.null(weights)) {
         # the argument names are:
         # - par for parameters (like in optim function)
         # - fix.arg for optional fixed parameters
@@ -198,8 +136,9 @@ mledist <- function (data, distr, start=NULL, fix.arg=NULL, optim.method="defaul
             }
         }
     }
-    else {# if !cens
-        argpdistname<-names(formals(pdistname))
+    else if(cens && is.null(weights)) #censored data
+    {    
+      argpdistname<-names(formals(pdistname))
         if (("log" %in% argddistname) & ("log.p" %in% argpdistname))
             fnobjcens <- function(par, fix.arg, rcens, lcens, icens, ncens, ddistnam, pdistnam)
                 -sum(do.call(ddistnam, c(list(x=ncens), as.list(par), as.list(fix.arg), list(log=TRUE)))) -
@@ -214,7 +153,25 @@ mledist <- function (data, distr, start=NULL, fix.arg=NULL, optim.method="defaul
                 sum(log(1-do.call(pdistnam, c(list(q=rcens), as.list(par), as.list(fix.arg))))) -
                 sum(log(do.call(pdistnam, c(list(q=icens$right), as.list(par), as.list(fix.arg))) - 
                 do.call(pdistnam, c(list(q=icens$left), as.list(par), as.list(fix.arg))) ))
-    }
+    }else if(!cens && !is.null(weights))
+    {
+        fnobj <- function(par, fix.arg, obs, ddistnam) {
+          -sum(weights * log(do.call(ddistnam, c(list(obs), as.list(par), as.list(fix.arg)) ) ) )
+        }
+    }else if(cens && !is.null(weights))
+    {
+      fnobjcens <- function(par, fix.arg, rcens, lcens, icens, ncens, ddistnam, pdistnam)
+      {
+        p1 <- log(do.call(ddistnam, c(list(x=ncens), as.list(par), as.list(fix.arg))))
+        p2 <- log(do.call(pdistnam, c(list(q=lcens), as.list(par), as.list(fix.arg)))) 
+        p3 <- log(1-do.call(pdistnam, c(list(q=rcens), as.list(par), as.list(fix.arg))))
+        p4 <- log(do.call(pdistnam, c(list(q=icens$right), as.list(par), as.list(fix.arg))) - 
+                    do.call(pdistnam, c(list(q=icens$left), as.list(par), as.list(fix.arg))) )
+        -sum(weights * p1) - sum(weights * p2) - sum(weights * p3) - sum(weights * p4) 
+      }
+    }else
+        stop("not yet implemented.")
+   
     # Choice of the optimization method    
     if (optim.method == "default")
     {
@@ -229,9 +186,12 @@ mledist <- function (data, distr, start=NULL, fix.arg=NULL, optim.method="defaul
     }else
         meth <- optim.method
         
+    owarn <- getOption("warn")
+    
     # Try to minimize the minus (log-)likelihood using the base R optim function
     if(is.null(custom.optim))
     {
+        options(warn=ifelse(silent, -1, 0))
         if (!cens)
             opttryerror <- try(opt <- optim(par=vstart, fn=fnobj, fix.arg=fix.arg, obs=data, 
                 ddistnam=ddistname, hessian=TRUE, method=meth, lower=lower, upper=upper, 
@@ -241,26 +201,30 @@ mledist <- function (data, distr, start=NULL, fix.arg=NULL, optim.method="defaul
                 rcens=rcens, lcens=lcens, icens=icens, ncens=ncens, ddistnam=ddistname, 
                 pdistnam=pdistname, hessian=TRUE, method=meth, lower=lower, upper=upper, 
                 ...), silent=TRUE)   
+        options(warn=owarn)
         
         if (inherits(opttryerror, "try-error"))
         {
-            warnings("The function optim encountered an error and stopped")
-            print(opttryerror)          
+            warnings("The function optim encountered an error and stopped.")
+            if(getOption("show.error.messages")) print(attr(opttryerror, "condition"))          
             return(list(estimate = rep(NA, length(vstart)), convergence = 100, loglik = NA, 
-                        hessian = NA))
+                        hessian = NA, optim.function="optim", fix.arg = fix.arg, 
+                        optim.method=meth, fix.arg.fun = fix.arg.fun))
         }
         
         if (opt$convergence>0) {
             warnings("The function optim failed to converge, with the error code ", 
                      opt$convergence)
-            return(list(estimate = rep(NA, length(vstart)), convergence = opt$convergence, 
-                        loglik = NA, hessian = NA, message = opt$message))
         }
+        if(is.null(names(opt$par)))
+          names(opt$par) <- names(vstart)
         res <- list(estimate = opt$par, convergence = opt$convergence, loglik = -opt$value, 
-                    hessian = opt$hessian, optim.function="optim", fix.arg = fix.arg)
+                    hessian = opt$hessian, optim.function="optim", fix.arg = fix.arg, 
+                    optim.method=meth, fix.arg.fun = fix.arg.fun, weights = weights)
     }
     else # Try to minimize the minus (log-)likelihood using a user-supplied optim function 
     {
+        options(warn=ifelse(silent, -1, 0))
         if (!cens)
             opttryerror <- try(opt <- custom.optim(fn=fnobj, fix.arg=fix.arg, obs=data, 
                 ddistnam=ddistname, par=vstart, ...), silent=TRUE)
@@ -268,24 +232,31 @@ mledist <- function (data, distr, start=NULL, fix.arg=NULL, optim.method="defaul
             opttryerror <-try(opt<-custom.optim(fn=fnobjcens, fix.arg=fix.arg, rcens=rcens, 
                 lcens=lcens, icens=icens, ncens=ncens, ddistnam=ddistname, pdistnam=pdistname, 
                 par=vstart, ...), silent=TRUE)              
+        options(warn=owarn)
         
         if (inherits(opttryerror, "try-error"))
         {
-            warnings("The customized optimization function encountered an error and stopped")
-            print(opttryerror)          
+            warnings("The customized optimization function encountered an error and stopped.")
+            if(getOption("show.error.messages")) print(attr(opttryerror, "condition"))          
             return(list(estimate = rep(NA, length(vstart)), convergence = 100, loglik = NA, 
-                        hessian = NA))
+                        hessian = NA, optim.function=custom.optim, fix.arg = fix.arg, fix.arg.fun = fix.arg.fun))
         }
         
         if (opt$convergence>0) {
             warnings("The customized optimization function failed to converge, with the error code ", 
                      opt$convergence)
-            return(list(estimate = rep(NA, length(vstart)), convergence = opt$convergence, 
-                        loglik = NA, hessian = NA, optim.function=custom.optim))
         }
+        argdot <- list(...)
+        method.cust <- argdot[argdot == "method"]
+        if(length(method.cust) == 0)
+        {
+          method.cust <- NULL
+        }
+        if(is.null(names(opt$par)))
+          names(opt$par) <- names(vstart)
         res <- list(estimate = opt$par, convergence = opt$convergence, loglik = -opt$value, 
-                    hessian = opt$hessian, optim.function=custom.optim, fix.arg = fix.arg)
-        
+                      hessian = opt$hessian, optim.function = custom.optim, fix.arg = fix.arg,
+                      method = method.cust, fix.arg.fun = fix.arg.fun, weights = weights)        
     }   
         
     return(res) 
@@ -294,5 +265,5 @@ mledist <- function (data, distr, start=NULL, fix.arg=NULL, optim.method="defaul
 ## old function with previous name for censored data
 mledistcens <- function(censdata, distr, start=NULL, optim.method="default", lower=-Inf, upper=Inf)
 {
-    stop("The function \"mledistcens\" is no more used. Now the same function \"mledist\" must be used for censored and non censored data.")
+    stop("The function \"mledistcens\" is no more used. Now the same function \"mledist\" must be used both for censored and non censored data.")
 }
