@@ -36,6 +36,7 @@ plotdist <- function(data, distr, para, histo = TRUE, breaks="default", demp = F
     s <- sort(data)
     n <- length(data)
     
+    
     if (missing(distr)) 
   { ## Plot of data only
         par(mfrow=c(1, 2))
@@ -136,7 +137,7 @@ plotdist <- function(data, distr, para, histo = TRUE, breaks="default", demp = F
         # plot of continuous data with theoretical distribution
             par(mfrow=c(2, 2))
             obsp <- ppoints(s)
-            theop <- do.call(pdistname, c(list(q=s), as.list(para)))
+            
             # plot of empirical and theoretical density
             # computes densities in order to define limits for y-axis
             if (breaks=="default")
@@ -145,7 +146,10 @@ plotdist <- function(data, distr, para, histo = TRUE, breaks="default", demp = F
                 h <- hist(data, breaks=breaks, plot=FALSE, ...)           
             xhist <- seq(min(h$breaks), max(h$breaks), length=1000)
             yhist <- do.call(ddistname, c(list(x=xhist), as.list(para)))
+            if(length(yhist) != length(xhist))
+              stop("problem when computing densities.")
             ymax <- ifelse(is.finite(max(yhist)), max(max(h$density), max(yhist)), max(h$density)) 
+            
             # PLOT 1 - plot of empirical and  theoretical density
             # empirical density
             if (histo)
@@ -170,6 +174,9 @@ plotdist <- function(data, distr, para, histo = TRUE, breaks="default", demp = F
            
             # PLOT 2 - plot of the qqplot
             theoq <- do.call(qdistname, c(list(p=obsp), as.list(para)))
+            if(length(theoq) != length(obsp))
+              stop("problem when computing quantities.")
+            
             plot(theoq, s, main=" Q-Q plot", xlab="Theoretical quantiles", 
 				      ylab="Empirical quantiles", ...)
             abline(0, 1)
@@ -177,6 +184,9 @@ plotdist <- function(data, distr, para, histo = TRUE, breaks="default", demp = F
             # PLOT 3 - plot of the cumulative probability distributions
             xmin <- h$breaks[1]
             xmax <- h$breaks[length(h$breaks)]
+				    if(length(s) != length(obsp))
+				      stop("problem when computing probabilities.")
+				      
             plot(s, obsp, main=paste("Empirical and theoretical CDFs"), xlab="Data", 
 				      ylab="CDF", xlim=c(xmin, xmax), ...)
             sfin <- seq(xmin, xmax, by=(xmax-xmin)/100)
@@ -184,7 +194,12 @@ plotdist <- function(data, distr, para, histo = TRUE, breaks="default", demp = F
             lines(sfin, theopfin, lty=1,col="red")
             
             # PLOT 4 - plot of the ppplot
-            plot(theop, obsp, main="P-P plot", xlab="Theoretical probabilities", 
+            
+				    theop <- do.call(pdistname, c(list(q=s), as.list(para)))
+				    if(length(theop) != length(obsp))
+				      stop("problem when computing probabilities.")
+				    
+				    plot(theop, obsp, main="P-P plot", xlab="Theoretical probabilities", 
 				      ylab="Empirical probabilities", ...)
             abline(0, 1)
         }
@@ -200,6 +215,9 @@ plotdist <- function(data, distr, para, histo = TRUE, breaks="default", demp = F
             xvalfin <- seq(min(xval), max(xval),by=1)
             xlinesdec <- min((max(xval)-min(xval))/30, 0.4)
             yd <- do.call(ddistname, c(list(x=xvalfin), as.list(para)))
+            if(length(yd) != length(xvalfin))
+              stop("problem when computing density points.")
+            
             ydobs <- as.vector(t)/n
             ydmax <- max(yd, ydobs)
             plot(xvalfin+xlinesdec, yd, type='h', xlim=c(min(xval), max(xval)+xlinesdec), 
@@ -213,6 +231,9 @@ plotdist <- function(data, distr, para, histo = TRUE, breaks="default", demp = F
             
             # plot of the cumulative probability distributions
             ycdf <- do.call(pdistname, c(list(q=xvalfin), as.list(para)))
+            if(length(ycdf) != length(xvalfin))
+              stop("problem when computing probabilities.")
+            
 			      plot(xvalfin, ycdf, type="s", xlim=c(min(xval), max(xval)+xlinesdec), 
 				        ylim=c(0, 1), lty=1, col="red", 
 				        main="Emp. and theo. CDFs", xlab="Data", 
